import numpy as np
import scipy
import matplotlib.pyplot as plt
from skimage import filters as skfilters
import plotly
import plotly.graph_objs as go

import os



# plot lines    
def plot_gallery_line(title, data, n_col, n_row):
    plt.figure(figsize=(2.0 * n_col, 2.26 * n_row))
    plt.suptitle(title, size=16)
    for i, comp in enumerate(data):
        plt.subplot(n_row, n_col, i + 1)
        vmax = max(comp.max(), -comp.min())
        plt.plot(
            comp
        )
        
        if i + 1 not in [k*n_col+1 for k in range(n_row)]:
            plt.xticks(())
            plt.yticks(())
            
    plt.subplots_adjust(0.01, 0.05, 0.99, 0.93, 0.04, 0.0)
    


def plot_unc_band(lp_hr, lp_rec, lp_rec_unc, fit_method):

    x_pltly = np.arange(len(lp_hr))


    fig = go.Figure([
        go.Scatter(
            name='HR (gt)',
            x=x_pltly,
            y=lp_hr,
            mode='lines',
            line=dict(color='rgb(0, 150, 255)'),
        ),
        go.Scatter(
            name=f"{fit_method}",
            x=x_pltly,
            y=lp_rec,
            mode='lines',
            line=dict(color='rgb(255,99,71)'),
        ),
        go.Scatter(
            name='Upper Bound',
            x=x_pltly,
            y=lp_rec + 2*lp_rec_unc,
            mode='lines',
            marker=dict(color="#DC143C"),
            line=dict(width=0),
            showlegend=False
        ),
        go.Scatter(
            name='Lower Bound',
            x=x_pltly,
            y=lp_rec - 2*lp_rec_unc,
            marker=dict(color="#DC143C"),
            line=dict(width=0),
            mode='lines',
            fillcolor='rgba(255,99,71, 0.3)',
            fill='tonexty',
            showlegend=False
        )
    ])
    fig.update_layout(
        yaxis_title='log intensity (a.u.)',
        xaxis_title='pixel/coord number (a.u.)',
        title=f"{fit_method} reconstruction with 95% confidence on uncertainities",
        hovermode="x"
    )
    fig.show()
    


def create_experiment_dirs(exp_dir):
    """
    Create Directories of a regular tensorflow experiment directory
    :param exp_dir:
    :return summary_dir, checkpoint_dir:
    """
    experiment_dir = "experiments/" + exp_dir + "/"
    summary_dir = experiment_dir + 'summaries/'
    checkpoint_dir = experiment_dir + 'checkpoints/'
    output_dir = experiment_dir + 'output/'
    test_dir = experiment_dir + 'test/'
    dirs = [summary_dir, checkpoint_dir, output_dir, test_dir]
    try:
        for dir_ in dirs:
            if not os.path.exists(dir_):
                os.makedirs(dir_)
        print("Experiment directories created")
        return experiment_dir, summary_dir, checkpoint_dir, output_dir, test_dir
    except Exception as err:
        print("Creating directories error: {0}".format(err))
        exit(-1) 


def save_train_test_h5(pes_train, pes_test, X_train, X_test, Y_train, Y_test, spec_raw_pe, data_info, exp_dir):
    import h5py

    hf = h5py.File(f"experiments/{exp_dir}/output/data.h5", 'w')
    hf.create_dataset('pes_train', data=pes_train)
    hf.create_dataset('pes_test', data=pes_test)
    hf.create_dataset('X_train', data=X_train)
    hf.create_dataset('X_test', data=X_test)
    hf.create_dataset('Y_train', data=Y_train)
    hf.create_dataset('Y_test', data=Y_test)
    hf.create_dataset('spec_raw_pe', data=spec_raw_pe[0,:])
    for key, val in data_info.items():
        hf.attrs[key] = val

 
    hf.close()

def save_rec_data_h5(sc_res, sc_res_unc, exp_dir):
    import h5py
    hf = h5py.File(f"experiments/{exp_dir}/output/rec_data.h5", 'w')
    hf.create_dataset('sc_res', data=sc_res)
    hf.create_dataset('sc_res_unc', data=sc_res_unc)
    hf.close()


def save_checkpoint(A_inf, b_inf, u_inf, Y_norm, exp_dir):

    import h5py
    hf = h5py.File(f"experiments/{exp_dir}/checkpoints/chp.h5", 'w')
    hf.create_dataset('A_inf', data=A_inf)
    hf.create_dataset('b_inf', data=b_inf)
    hf.create_dataset('u_inf', data=u_inf)
    hf.create_dataset('Y_norm', data=Y_norm)
    hf.close()

def save_chp_bfgs(model, exp_dir):
    import h5py
    hf = h5py.File(f"experiments/{exp_dir}/checkpoints/chp.h5", 'w')

    for k, v in model.items():
        hf.create_dataset(k, data=v)
    hf.close()



def save_checkpoint_fft(A_inf_1, b_inf_1,
                        A_inf_2, b_inf_2, 
                        u_inf_1, u_inf_2,
                        Y_norm, exp_dir):

    import h5py
    hf = h5py.File(f"experiments/{exp_dir}/checkpoints/chp.h5", 'w')
    hf.create_dataset('A_inf_1', data=A_inf_1)
    hf.create_dataset('b_inf_1', data=b_inf_1)
    hf.create_dataset('A_inf_2', data=A_inf_2)
    hf.create_dataset('b_inf_2', data=b_inf_2)
    hf.create_dataset('u_inf_1', data=u_inf_1)
    hf.create_dataset('u_inf_2', data=u_inf_2)
    hf.create_dataset('Y_norm', data=Y_norm)
    hf.close()



def load_train_test_h5(exp_dir):
    import h5py
    # Load Data
    filename = f"experiments/{exp_dir}/output/data.h5"
    file = h5py.File(filename, 'r')
    pes_train = file["pes_train"][()]  # returns as a numpy array
    pes_test = file["pes_test"][()]  # returns as a numpy array
    spec_train = file["spec_train"][()]  # returns as a numpy array
    spec_test = file["spec_test"][()]  # returns as a numpy array
    X_train = file["X_train"][()]  # returns as a numpy array
    X_test = file["X_test"][()]  # returns as a numpy array
    Y_train = file["Y_train"][()]  # returns as a numpy array
    Y_test = file["Y_test"][()]  # returns as a numpy array
    spec_raw_pe = file["spec_raw_pe"][()]  # returns as a numpy 
    xgm_pulseen_train = file["xgm_pulseen_train"][()]  
    xgm_pulseen_test = file["xgm_pulseen_test"][()]  

    att_names = file.attrs.keys()
    att_dict = {}
    for att_name in att_names:
        att_dict[att_name] = file.attrs[att_name]

    return Y_train, Y_test, spec_train, spec_test, spec_raw_pe, X_train, X_test, pes_train, pes_test,  att_dict, xgm_pulseen_train, xgm_pulseen_test


def load_rec_data_h5(exp_dir):
    import h5py
    filename = f"experiments/{exp_dir}/output/pred_data.h5"
    file = h5py.File(filename, 'r')
    sc_res = file["res"][()]  # returns as a numpy array
    sc_res_unc = file["res_unc"][()]  # returns as a numpy array

    return sc_res, sc_res_unc

def load_rec_data_h5_bfgs_pca(exp_dir):
    import h5py
    filename = f"experiments/{exp_dir}/output/pred_data.h5"
    file = h5py.File(filename, 'r')
    sc_res = file["res"][()]  # returns as a numpy array
    sc_res_unc = file["res_unc"][()]  # returns as a numpy array
    sc_res_unc_specpca = file["res_unc_specpca"][()]  # returns as a numpy array
    sc_res_unc_total = file["res_unc_total"][()]  # returns as a numpy array
   

    return sc_res, sc_res_unc, sc_res_unc_specpca, sc_res_unc_total

def load_rec_data_h5_bfgs_pca_eps(exp_dir):
    import h5py
    filename = f"experiments/{exp_dir}/output/pred_data.h5"
    file = h5py.File(filename, 'r')
    sc_res = file["res"][()]  # returns as a numpy array
    sc_res_unc = file["res_unc"][()]  # returns as a numpy array
    sc_res_unc_specpca = file["res_unc_specpca"][()]  # returns as a numpy array
    sc_res_unc_total = file["res_unc_total"][()]  # returns as a numpy array
    sc_res_eps = file["res_eps"][()]  # returns as a numpy array

    return sc_res, sc_res_unc, sc_res_unc_specpca, sc_res_unc_total, sc_res_eps

def load_checkpoint(exp_dir):
    import h5py
    # Load Checkpoint Data
    filename = f"experiments/{exp_dir}/checkpoints/chp.h5"
    file = h5py.File(filename, 'r')
    A_inf = file["A_inf"][()]  # returns as a numpy array
    b_inf = file["b_inf"][()]  # returns as a numpy array
    u_inf = file["u_inf"][()]  # returns as a numpy array
    Y_norm = file["Y_norm"][()]  # returns as a numpy array

    return A_inf, b_inf, u_inf, Y_norm


